@@ -1,14 +1,11 @@
 import asyncio
+import ssl
 
 try:
     import aiohttp
 except ImportError:  # pragma: no cover
     aiohttp = None
 import six
-try:
-    import websockets
-except ImportError:  # pragma: no cover
-    websockets = None
 
 from . import client
 from . import exceptions
@@ -30,6 +27,12 @@ class AsyncClient(client.Client):
                  packets. Custom json modules must have ``dumps`` and ``loads``
                  functions that are compatible with the standard library
                  versions.
+    :param request_timeout: A timeout in seconds for requests. The default is
+                            5 seconds.
+    :param ssl_verify: ``True`` to verify SSL certificates, or ``False`` to
+                       skip SSL certificate verification, allowing
+                       connections to servers with self signed certificates.
+                       The default is ``True``.
     """
     def is_asyncio_based(self):
         return True
@@ -158,6 +161,11 @@ class AsyncClient(client.Client):
         """Create an event object."""
         return asyncio.Event()
 
+    def _reset(self):
+        if self.http:  # pragma: no cover
+            asyncio.ensure_future(self.http.close())
+        super()._reset()
+
     async def _connect_polling(self, url, headers, engineio_path):
         """Establish a long-polling connection to the Engine.IO server."""
         if aiohttp is None:  # pragma: no cover
@@ -167,12 +175,13 @@ class AsyncClient(client.Client):
         self.base_url = self._get_engineio_url(url, engineio_path, 'polling')
         self.logger.info('Attempting polling connection to ' + self.base_url)
         r = await self._send_request(
-            'GET', self.base_url + self._get_url_timestamp(), headers=headers)
+            'GET', self.base_url + self._get_url_timestamp(), headers=headers,
+            timeout=self.request_timeout)
         if r is None:
             self._reset()
             raise exceptions.ConnectionError(
                 'Connection refused by the server')
-        if r.status != 200:
+        if r.status < 200 or r.status >= 300:
             raise exceptions.ConnectionError(
                 'Unexpected status code {} in server response'.format(
                     r.status))
@@ -214,8 +223,8 @@ class AsyncClient(client.Client):
 
     async def _connect_websocket(self, url, headers, engineio_path):
         """Establish or upgrade to a WebSocket connection with the server."""
-        if websockets is None:  # pragma: no cover
-            self.logger.error('websockets package not installed')
+        if aiohttp is None:  # pragma: no cover
+            self.logger.error('aiohttp package not installed')
             return False
         websocket_url = self._get_engineio_url(url, engineio_path,
                                                'websocket')
@@ -229,12 +238,24 @@ class AsyncClient(client.Client):
             self.base_url = websocket_url
             self.logger.info(
                 'Attempting WebSocket connection to ' + websocket_url)
+
+        if self.http is None or self.http.closed:  # pragma: no cover
+            self.http = aiohttp.ClientSession()
+
         try:
-            ws = await websockets.connect(
-                websocket_url + self._get_url_timestamp(),
-                extra_headers=headers)
-        except (websockets.exceptions.InvalidURI,
-                websockets.exceptions.InvalidHandshake):
+            if not self.ssl_verify:
+                ssl_context = ssl.create_default_context()
+                ssl_context.check_hostname = False
+                ssl_context.verify_mode = ssl.CERT_NONE
+                ws = await self.http.ws_connect(
+                    websocket_url + self._get_url_timestamp(),
+                    headers=headers, ssl=ssl_context)
+            else:
+                ws = await self.http.ws_connect(
+                    websocket_url + self._get_url_timestamp(),
+                    headers=headers)
+        except (aiohttp.client_exceptions.WSServerHandshakeError,
+                aiohttp.client_exceptions.ServerConnectionError):
             if upgrade:
                 self.logger.warning(
                     'WebSocket upgrade failed: connection error')
@@ -245,14 +266,14 @@ class AsyncClient(client.Client):
             p = packet.Packet(packet.PING, data='probe').encode(
                 always_bytes=False)
             try:
-                await ws.send(p)
+                await ws.send_str(p)
             except Exception as e:  # pragma: no cover
                 self.logger.warning(
                     'WebSocket upgrade failed: unexpected send exception: %s',
                     str(e))
                 return False
             try:
-                p = await ws.recv()
+                p = (await ws.receive()).data
             except Exception as e:  # pragma: no cover
                 self.logger.warning(
                     'WebSocket upgrade failed: unexpected recv exception: %s',
@@ -265,19 +286,17 @@ class AsyncClient(client.Client):
                 return False
             p = packet.Packet(packet.UPGRADE).encode(always_bytes=False)
             try:
-                await ws.send(p)
+                await ws.send_str(p)
             except Exception as e:  # pragma: no cover
                 self.logger.warning(
                     'WebSocket upgrade failed: unexpected send exception: %s',
                     str(e))
                 return False
             self.current_transport = 'websocket'
-            if self.http:  # pragma: no cover
-                await self.http.close()
             self.logger.info('WebSocket upgrade was successful')
         else:
             try:
-                p = await ws.recv()
+                p = (await ws.receive()).data
             except Exception as e:  # pragma: no cover
                 raise exceptions.ConnectionError(
                     'Unexpected recv exception: ' + str(e))
@@ -314,6 +333,8 @@ class AsyncClient(client.Client):
             await self._trigger_event('message', pkt.data, run_async=True)
         elif pkt.packet_type == packet.PONG:
             self.pong_received = True
+        elif pkt.packet_type == packet.CLOSE:
+            await self.disconnect(abort=True)
         elif pkt.packet_type == packet.NOOP:
             pass
         else:
@@ -331,14 +352,25 @@ class AsyncClient(client.Client):
             pkt.data if not isinstance(pkt.data, bytes) else '<binary>')
 
     async def _send_request(
-            self, method, url, headers=None, body=None):  # pragma: no cover
+            self, method, url, headers=None, body=None,
+            timeout=None):  # pragma: no cover
         if self.http is None or self.http.closed:
             self.http = aiohttp.ClientSession()
-        method = getattr(self.http, method.lower())
+        http_method = getattr(self.http, method.lower())
+
         try:
-            return await method(url, headers=headers, data=body)
-        except aiohttp.ClientError:
-            return
+            if not self.ssl_verify:
+                return await http_method(
+                    url, headers=headers, data=body,
+                    timeout=aiohttp.ClientTimeout(total=timeout), ssl=False)
+            else:
+                return await http_method(
+                    url, headers=headers, data=body,
+                    timeout=aiohttp.ClientTimeout(total=timeout))
+
+        except (aiohttp.ClientError, asyncio.TimeoutError) as exc:
+            self.logger.info('HTTP %s request to %s failed with error %s.',
+                             method, url, exc)
 
     async def _trigger_event(self, event, *args, **kwargs):
         """Invoke an event handler."""
@@ -382,10 +414,13 @@ class AsyncClient(client.Client):
         interval.
         """
         self.pong_received = True
-        self.ping_loop_event.clear()
+        if self.ping_loop_event is None:
+            self.ping_loop_event = self.create_event()
+        else:
+            self.ping_loop_event.clear()
         while self.state == 'connected':
             if not self.pong_received:
-                self.logger.warning(
+                self.logger.info(
                     'PONG response has not been received, aborting')
                 if self.ws:
                     await self.ws.close()
@@ -407,13 +442,14 @@ class AsyncClient(client.Client):
             self.logger.info(
                 'Sending polling GET request to ' + self.base_url)
             r = await self._send_request(
-                'GET', self.base_url + self._get_url_timestamp())
+                'GET', self.base_url + self._get_url_timestamp(),
+                timeout=max(self.ping_interval, self.ping_timeout) + 5)
             if r is None:
                 self.logger.warning(
                     'Connection refused by the server, aborting')
                 await self.queue.put(None)
                 break
-            if r.status != 200:
+            if r.status < 200 or r.status >= 300:
                 self.logger.warning('Unexpected status code %s in server '
                                     'response, aborting', r.status)
                 await self.queue.put(None)
@@ -431,7 +467,8 @@ class AsyncClient(client.Client):
         self.logger.info('Waiting for write loop task to end')
         await self.write_loop_task
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         await self.ping_loop_task
         if self.state == 'connected':
             await self._trigger_event('disconnect', run_async=False)
@@ -447,9 +484,11 @@ class AsyncClient(client.Client):
         while self.state == 'connected':
             p = None
             try:
-                p = await self.ws.recv()
-            except websockets.exceptions.ConnectionClosed:
-                self.logger.warning(
+                p = (await self.ws.receive()).data
+                if p is None:  # pragma: no cover
+                    raise RuntimeError('WebSocket read returned None')
+            except aiohttp.client_exceptions.ServerDisconnectedError:
+                self.logger.info(
                     'Read loop: WebSocket connection was closed, aborting')
                 await self.queue.put(None)
                 break
@@ -466,7 +505,8 @@ class AsyncClient(client.Client):
         self.logger.info('Waiting for write loop task to end')
         await self.write_loop_task
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         await self.ping_loop_task
         if self.state == 'connected':
             await self._trigger_event('disconnect', run_async=False)
@@ -492,7 +532,6 @@ class AsyncClient(client.Client):
             except (self.queue.Empty, asyncio.TimeoutError,
                     asyncio.CancelledError):
                 self.logger.error('packet queue is empty, aborting')
-                self._reset()
                 break
             if packets == [None]:
                 self.queue.task_done()
@@ -514,14 +553,15 @@ class AsyncClient(client.Client):
                 p = payload.Payload(packets=packets)
                 r = await self._send_request(
                     'POST', self.base_url, body=p.encode(),
-                    headers={'Content-Type': 'application/octet-stream'})
+                    headers={'Content-Type': 'application/octet-stream'},
+                    timeout=self.request_timeout)
                 for pkt in packets:
                     self.queue.task_done()
                 if r is None:
                     self.logger.warning(
                         'Connection refused by the server, aborting')
                     break
-                if r.status != 200:
+                if r.status < 200 or r.status >= 300:
                     self.logger.warning('Unexpected status code %s in server '
                                         'response, aborting', r.status)
                     self._reset()
@@ -530,10 +570,15 @@ class AsyncClient(client.Client):
                 # websocket
                 try:
                     for pkt in packets:
-                        await self.ws.send(pkt.encode(always_bytes=False))
+                        if pkt.binary:
+                            await self.ws.send_bytes(pkt.encode(
+                                always_bytes=False))
+                        else:
+                            await self.ws.send_str(pkt.encode(
+                                always_bytes=False))
                         self.queue.task_done()
-                except websockets.exceptions.ConnectionClosed:
-                    self.logger.warning(
+                except aiohttp.client_exceptions.ServerDisconnectedError:
+                    self.logger.info(
                         'Write loop: WebSocket connection was closed, '
                         'aborting')
                     break
