@@ -43,10 +43,10 @@ def signal_handler(sig, frame):
         return original_signal_handler(sig, frame)
     else:  # pragma: no cover
         # Handle case where no original SIGINT handler was present.
-        return signal.def_int_handler(sig, frame)
+        return signal.default_int_handler(sig, frame)
 
 
-original_signal_handler = signal.signal(signal.SIGINT, signal_handler)
+original_signal_handler = None
 
 
 class Client(object):
@@ -64,10 +64,10 @@ class Client(object):
                  versions.
     :param request_timeout: A timeout in seconds for requests. The default is
                             5 seconds.
-    :param ssl_verify: ``True`` if SSL connections should be fully verified or
-                       ``False`` to skip SSL certificate verification allowing
-                       connection to server with self signed certificates. The
-                       default is ``True``
+    :param ssl_verify: ``True`` to verify SSL certificates, or ``False`` to
+                       skip SSL certificate verification, allowing
+                       connections to servers with self signed certificates.
+                       The default is ``True``.
     """
     event_names = ['connect', 'disconnect', 'message']
 
@@ -76,6 +76,10 @@ class Client(object):
                  json=None,
                  request_timeout=5,
                  ssl_verify=True):
+        global original_signal_handler
+        if original_signal_handler is None:
+            original_signal_handler = signal.signal(signal.SIGINT,
+                                                    signal_handler)
         self.handlers = {}
         self.base_url = None
         self.transports = None
@@ -90,7 +94,7 @@ class Client(object):
         self.read_loop_task = None
         self.write_loop_task = None
         self.ping_loop_task = None
-        self.ping_loop_event = self.create_event()
+        self.ping_loop_event = None
         self.queue = None
         self.state = 'disconnected'
         self.ssl_verify = ssl_verify
@@ -364,7 +368,7 @@ class Client(object):
                 ws = websocket.create_connection(
                     websocket_url + self._get_url_timestamp(), header=headers,
                     cookie=cookies)
-        except ConnectionError:
+        except (ConnectionError, IOError, websocket.WebSocketException):
             if upgrade:
                 self.logger.warning(
                     'WebSocket upgrade failed: connection error')
@@ -515,13 +519,16 @@ class Client(object):
         interval.
         """
         self.pong_received = True
-        self.ping_loop_event.clear()
+        if self.ping_loop_event is None:
+            self.ping_loop_event = self.create_event()
+        else:
+            self.ping_loop_event.clear()
         while self.state == 'connected':
             if not self.pong_received:
                 self.logger.info(
                     'PONG response has not been received, aborting')
                 if self.ws:
-                    self.ws.shutdown()
+                    self.ws.close(timeout=0)
                 self.queue.put(None)
                 break
             self.pong_received = False
@@ -560,7 +567,8 @@ class Client(object):
         self.logger.info('Waiting for write loop task to end')
         self.write_loop_task.join()
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         self.ping_loop_task.join()
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
@@ -595,7 +603,8 @@ class Client(object):
         self.logger.info('Waiting for write loop task to end')
         self.write_loop_task.join()
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         self.ping_loop_task.join()
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
